#include <stdio.h>
#include <string.h>

#include <atptypes.h>
#include <at_if.h>

#include <atpflashapi.h>

void printHelp()
{
    printf("*** sphfreq v1.0.3   (c)2016 @speedUpLoop / emkay ***\n\n");
    printf("Setting LTE-Band / Frequency (or LED-State) of Telekom-Speedport-Hybrid\n");    
    printf("Usage:  sphfreq [show/watch/all/800/1800/2600/low/high]\n");
    printf("Options:\n");
    printf("        show\tshow current lte-config\n");
    printf("        watch\tsimilar to 'show' - with auto-update\n");
    printf("-------------------------------------------------------------------------------\n");
    printf("        all\tallow  800MHz, 1800MHz, 2600MHz\t--> hex-mask 80044\n");
    printf("        800\tallow  800MHz\t\t\t--> hex-mask 80000\n");
    printf("        1800\tallow 1800MHz\t\t\t--> hex-mask 4\n");
    printf("        2600\tallow 2600MHz\t\t\t--> hex-mask 40\n");
    printf("        low\tallow  800MHz, 1800MHz\t\t--> hex-mask 80004\n");
    printf("        high\tallow 1800MHz, 2600MHz\t\t--> hex-mask 44\n\n");
    printf("--LEDs-------------------------------------------------------------------------\n");
    printf("Usage:  sphfreq leds all [off/hardoff]\n");
    printf("Options:\n");
    printf("        off\tturn all LEDs off\n");
    printf("        hardoff\tdeactivate all LEDs until reboot\n");
    printf("Usage:  sphfreq leds [power/red/lteall/lte1/lte2/lte3/lte4/lte5] [on/blink/off]\n");
    printf("Usage:  sphfreq leds [dsl/online/voip/dect] [on/blink/off]\n");
    printf("Usage:  sphfreq leds [service1/service2/wlan] [on/off]\n");
    printf("Usage:  sphfreq leds [wps] [blink/off]\n");    
    printf("        turn LED on or off - or let it blink...\n");       
}

AT_AMSS_RES_EN printCurrentCfg()
{
    NetCfgExResType rdata={0};
    AT_SysConfigEx(AT_CMD_TYPE_CHECK,NULL,&rdata);
    
    printf(">> Current-LTE-Config <<\n");
    printf("Acgorder:\t%s\n",strlen(rdata.Acqorder)>0?rdata.Acqorder:"-");
    printf("Band:\t\t%s\n",strlen(rdata.Band)>0?rdata.Band:"-");
    printf("Roam:\t\t%s\n",strlen(rdata.Roam)>0?rdata.Roam:"-");
    printf("Srvdomain:\t%s\n",strlen(rdata.Srvdomain)>0?rdata.Srvdomain:"-");
    printf("LTEBand:\t%s\n",strlen(rdata.LTEBand)>0?rdata.LTEBand:"-");
    
    return rdata.AtExeResult;
}

AT_AMSS_RES_EN printCurrentCellInfo()
{
    LcellinfoResType rdata={0};
    AT_Lcellinfo(AT_CMD_TYPE_CHECK,NULL,&rdata);
    
    printf(">> Current-CellInfo <<\n");
    printf("Freq:\t\t%s\n",strlen(rdata.Freq)>0?rdata.Freq:"-");
    printf("Band:\t\t%s\n",strlen(rdata.Band)>0?rdata.Band:"-");
    printf("PhyCellId:\t%s\n",strlen(rdata.CellId)>0?rdata.CellId:"-");
    printf("RSRP:\t\t%s\n",strlen(rdata.Rsrp)>0?rdata.Rsrp:"-");
    printf("RSRQ:\t\t%s\n",strlen(rdata.Rsrq)>0?rdata.Rsrq:"-");
    printf("RSSI:\t\t%s\n",strlen(rdata.Rssi)>0?rdata.Rssi:"-");
    
    return rdata.AtExeResult;    
}

AT_AMSS_RES_EN watchCellInfo()
{
    LcellinfoResType r1data={0};
    LocinfoResType r2data={0};
    printf(">> Current-CellInfo <<\n");
    while(1)
    {   
        AT_Lcellinfo(AT_CMD_TYPE_CHECK,NULL,&r1data);
        AT_Locinfo(AT_CMD_TYPE_CHECK,&r2data);
        if(r1data.AtExeResult==AT_CMD_RES_SUCCESS&&r2data.AtExeResult==AT_CMD_RES_SUCCESS)
        {
            printf("\r");        
            printf("Freq: %s * ",strlen(r1data.Freq)>0?r1data.Freq:"-");
            printf("Band: %s * ",strlen(r1data.Band)>0?r1data.Band:"-");
            printf("Cell: %s/%d * ",strlen(r1data.CellId)>0?r1data.CellId:"-",strtol(r2data.cell_id,NULL,0)>0?strtol(r2data.cell_id,NULL,0):"-");
            printf("RSRP: %4s * ",strlen(r1data.Rsrp)>0?r1data.Rsrp:"-");
            printf("RSRQ: %3s * ",strlen(r1data.Rsrq)>0?r1data.Rsrq:"-");
            printf("RSSI: %4s",strlen(r1data.Rssi)>0?r1data.Rssi:"-");
            fflush(stdout);
        }
    }    
    return r1data.AtExeResult;    
}

AT_AMSS_RES_EN printCurrentLocInfo()
{
    LocinfoResType rdata={0};
    AT_Locinfo(AT_CMD_TYPE_CHECK,&rdata);

    printf(">> Current-LocInfo <<\n");
    printf("PLMN:\t\t%s\n",strlen(rdata.plmn)>0?rdata.plmn:"-");
    printf("LAC:\t\t%d\n",strtol(rdata.lac,NULL,0)>0?strtol(rdata.lac,NULL,0):"-");
    printf("RAC:\t\t%d\n",strtol(rdata.rac,NULL,0)>0?strtol(rdata.rac,NULL,0):"-");
    printf("CellId:\t\t%d\n",strtol(rdata.cell_id,NULL,0)>0?strtol(rdata.cell_id,NULL,0):"-");
    
    return rdata.AtExeResult;
}

AT_AMSS_RES_EN setCurrentCfg(const VOS_CHAR* lteband)
{
    NetCfgExSetType sdata={0};
    NetCfgExResType rdata={0};
    printf("Setting new LTE-Config...");
    strncpy(sdata.Acqorder,"99",sizeof(sdata.Acqorder));
    strncpy(sdata.Band,"3FFFFFFF",sizeof(sdata.Band));
    strncpy(sdata.Roam,"3",sizeof(sdata.Roam));
    strncpy(sdata.Srvdomain,"4",sizeof(sdata.Srvdomain));
    strncpy(sdata.LTEBand,lteband,sizeof(sdata.LTEBand));
    AT_SysConfigEx(AT_CMD_TYPE_EXECUTE,&sdata,&rdata);
    if(rdata.AtExeResult==AT_CMD_RES_SUCCESS) printf("OK\n");
    else printf("Error\n");

    return rdata.AtExeResult;
}

void printAll()
{
    printCurrentCfg();
    printf("\n");
    printCurrentCellInfo();
    printf("\n");
    printCurrentLocInfo();    
}

void ledsOff()
{
    int i=0;
    for(i=BHAL_LED_Power;i<BHAL_LED_End;i++)
    {
        ATP_BHAL_SetLedState(i,BHAL_LED_STATE_OFF);
    }
}

int main(int argc, char **argv)
{
    AT_AMSS_RES_EN ret;
    VOS_CHAR TARGET_BAND[64]="80044";    
    const VOS_CHAR LTE_ALL[64]="80044";
    const VOS_CHAR LTE_800[64]="80000";
    const VOS_CHAR LTE_1800[64]="4";
    const VOS_CHAR LTE_2600[64]="40";
    const VOS_CHAR LTE_LOW[64]="80004";
    const VOS_CHAR LTE_HIGH[64]="44";
    
    if(argc==2)
    {
        if(strcmp(argv[1],"show")==0)
        {
            printAll();
            return 0;
        }
        else if(strcmp(argv[1],"watch")==0)
        {
            printCurrentCfg();
            printf("\n");
            watchCellInfo();
            return 0;
        }
        else if(strcmp(argv[1],"all")==0) strncpy(TARGET_BAND,LTE_ALL,sizeof(TARGET_BAND));
        else if(strcmp(argv[1],"800")==0) strncpy(TARGET_BAND,LTE_800,sizeof(TARGET_BAND));
        else if(strcmp(argv[1],"1800")==0) strncpy(TARGET_BAND,LTE_1800,sizeof(TARGET_BAND));
        else if(strcmp(argv[1],"2600")==0) strncpy(TARGET_BAND,LTE_2600,sizeof(TARGET_BAND));
        else if(strcmp(argv[1],"low")==0) strncpy(TARGET_BAND,LTE_LOW,sizeof(TARGET_BAND));
        else if(strcmp(argv[1],"high")==0) strncpy(TARGET_BAND,LTE_HIGH,sizeof(TARGET_BAND));
        else
        {
            printHelp();
            return 1;
        }
        ret=setCurrentCfg(TARGET_BAND);
        printf("\n");
        printAll();
    }
    else if(argc==4)
    {
        if(strcmp(argv[1],"leds")==0)
        {
            if(strcmp(argv[2],"all")==0)
            {
                if(strcmp(argv[3],"hardoff")==0) ATP_EQUIP_AllLED_Off();
                else if(strcmp(argv[3],"off")==0) ledsOff();
                else printHelp();
            }
            else if(strcmp(argv[2],"power")==0)
            {
                if(strcmp(argv[3],"on")==0) ATP_BHAL_SetLedState(BHAL_LED_Power,BHAL_LED_STATE_ON);
                else if(strcmp(argv[3],"off")==0) ATP_BHAL_SetLedState(BHAL_LED_Power,BHAL_LED_STATE_OFF);
                else if(strcmp(argv[3],"blink")==0) ATP_BHAL_SetLedState(BHAL_LED_Power,BHAL_LED_STATE_SLOWBLINK_CONTINUES);
                else printHelp();
            }
            else if(strcmp(argv[2],"red")==0)
            {
                if(strcmp(argv[3],"on")==0) ATP_BHAL_SetLedState(BHAL_LED_Powerfail,BHAL_LED_STATE_ON);
                else if(strcmp(argv[3],"off")==0) ATP_BHAL_SetLedState(BHAL_LED_Powerfail,BHAL_LED_STATE_OFF);
                else if(strcmp(argv[3],"blink")==0) ATP_BHAL_SetLedState(BHAL_LED_Powerfail,BHAL_LED_STATE_SLOWBLINK_CONTINUES);
                else printHelp();
            }
            else if(strcmp(argv[2],"lteall")==0)
            {
                if(strcmp(argv[3],"on")==0) ATP_BHAL_SetLedState(BHAL_LED_LineLteAll,BHAL_LED_STATE_ON);
                else if(strcmp(argv[3],"off")==0) ATP_BHAL_SetLedState(BHAL_LED_LineLteAll,BHAL_LED_STATE_OFF);
                else if(strcmp(argv[3],"blink")==0) ATP_BHAL_SetLedState(BHAL_LED_LineLteAll,BHAL_LED_STATE_SLOWBLINK_CONTINUES);
                else printHelp();
            }
            else if(strcmp(argv[2],"lte1")==0)
            {
                if(strcmp(argv[3],"on")==0) ATP_BHAL_SetLedState(BHAL_LED_LineLte1,BHAL_LED_STATE_ON);
                else if(strcmp(argv[3],"off")==0) ATP_BHAL_SetLedState(BHAL_LED_LineLte1,BHAL_LED_STATE_OFF);
                else if(strcmp(argv[3],"blink")==0) ATP_BHAL_SetLedState(BHAL_LED_LineLte1,BHAL_LED_STATE_SLOWBLINK_CONTINUES);
                else printHelp();
            }
            else if(strcmp(argv[2],"lte2")==0)
            {
                if(strcmp(argv[3],"on")==0) ATP_BHAL_SetLedState(BHAL_LED_LineLte2,BHAL_LED_STATE_ON);
                else if(strcmp(argv[3],"off")==0) ATP_BHAL_SetLedState(BHAL_LED_LineLte2,BHAL_LED_STATE_OFF);
                else if(strcmp(argv[3],"blink")==0) ATP_BHAL_SetLedState(BHAL_LED_LineLte2,BHAL_LED_STATE_SLOWBLINK_CONTINUES);
                else printHelp();
            }
            else if(strcmp(argv[2],"lte3")==0)
            {
                if(strcmp(argv[3],"on")==0) ATP_BHAL_SetLedState(BHAL_LED_LineLte3,BHAL_LED_STATE_ON);
                else if(strcmp(argv[3],"off")==0) ATP_BHAL_SetLedState(BHAL_LED_LineLte3,BHAL_LED_STATE_OFF);
                else if(strcmp(argv[3],"blink")==0) ATP_BHAL_SetLedState(BHAL_LED_LineLte3,BHAL_LED_STATE_SLOWBLINK_CONTINUES);
                else printHelp();
            }
            else if(strcmp(argv[2],"lte4")==0)
            {
                if(strcmp(argv[3],"on")==0) ATP_BHAL_SetLedState(BHAL_LED_LineLte4,BHAL_LED_STATE_ON);
                else if(strcmp(argv[3],"off")==0) ATP_BHAL_SetLedState(BHAL_LED_LineLte4,BHAL_LED_STATE_OFF);
                else if(strcmp(argv[3],"blink")==0) ATP_BHAL_SetLedState(BHAL_LED_LineLte4,BHAL_LED_STATE_SLOWBLINK_CONTINUES);
                else printHelp();
            }
            else if(strcmp(argv[2],"lte5")==0)
            {
                if(strcmp(argv[3],"on")==0) ATP_BHAL_SetLedState(BHAL_LED_LineLte5,BHAL_LED_STATE_ON);
                else if(strcmp(argv[3],"off")==0) ATP_BHAL_SetLedState(BHAL_LED_LineLte5,BHAL_LED_STATE_OFF);
                else if(strcmp(argv[3],"blink")==0) ATP_BHAL_SetLedState(BHAL_LED_LineLte5,BHAL_LED_STATE_SLOWBLINK_CONTINUES);
                else printHelp();
            }
//////
            else if(strcmp(argv[2],"dsl")==0)
            {
                if(strcmp(argv[3],"on")==0) ATP_BHAL_SetLedState(BHAL_LED_Wan,BHAL_LED_STATE_OFF); 
                else if(strcmp(argv[3],"off")==0) ATP_BHAL_SetLedState(BHAL_LED_Wan,BHAL_LED_STATE_SLOWBLINK_CONTINUES);
                else if(strcmp(argv[3],"blink")==0) ATP_BHAL_SetLedState(BHAL_LED_Wan,BHAL_LED_STATE_ON);
                else printHelp();
            }
            else if(strcmp(argv[2],"online")==0)
            {
                if(strcmp(argv[3],"on")==0) ATP_BHAL_SetLedState(BHAL_LED_Internet,BHAL_LED_STATE_RED);
                else if(strcmp(argv[3],"off")==0) ATP_BHAL_SetLedState(BHAL_LED_Internet,BHAL_LED_STATE_OFF);
                else if(strcmp(argv[3],"blink")==0) ATP_BHAL_SetLedState(BHAL_LED_Internet,BHAL_LED_STATE_ON);
                else printHelp();
            }
            else if(strcmp(argv[2],"voip")==0)
            {
                if(strcmp(argv[3],"on")==0) ATP_BHAL_SetLedState(BHAL_LED_Voip1,BHAL_LED_STATE_ON);
                else if(strcmp(argv[3],"off")==0) ATP_BHAL_SetLedState(BHAL_LED_Voip1,BHAL_LED_STATE_SLOWBLINK_CONTINUES);
                else if(strcmp(argv[3],"blink")==0) ATP_BHAL_SetLedState(BHAL_LED_Voip1,BHAL_LED_STATE_OFF);
                else printHelp();
            }
            else if(strcmp(argv[2],"wlan")==0)
            {
                if(strcmp(argv[3],"on")==0) ATP_BHAL_SetLedState(BHAL_LED_Wlan,BHAL_LED_STATE_ON);
                else if(strcmp(argv[3],"off")==0) ATP_BHAL_SetLedState(BHAL_LED_Wlan,BHAL_LED_STATE_OFF);
//                else if(strcmp(argv[3],"blink")==0) ATP_BHAL_SetLedState(BHAL_LED_Wlan,BHAL_LED_STATE_SLOWBLINK_CONTINUES); //// doesn't work - todo
                else printHelp();
            }
            else if(strcmp(argv[2],"wps")==0)
            {
//                if(strcmp(argv[3],"on")==0) ATP_BHAL_SetLedState(BHAL_LED_Wps,BHAL_LED_STATE_ON); //// doesn't work - todo
                if(strcmp(argv[3],"off")==0) ATP_BHAL_SetLedState(BHAL_LED_Wps,BHAL_LED_STATE_OFF); 
                else if(strcmp(argv[3],"blink")==0) ATP_BHAL_SetLedState(BHAL_LED_Wps,BHAL_LED_STATE_SLOWBLINK_CONTINUES);
                else printHelp();
            }
            else if(strcmp(argv[2],"dect")==0)
            {
                if(strcmp(argv[3],"on")==0) ATP_BHAL_SetLedState(BHAL_LED_DectSearch,BHAL_LED_STATE_ON);
                else if(strcmp(argv[3],"off")==0) ATP_BHAL_SetLedState(BHAL_LED_DectSearch,BHAL_LED_STATE_OFF);
                else if(strcmp(argv[3],"blink")==0) ATP_BHAL_SetLedState(BHAL_LED_DectSearch,BHAL_LED_STATE_SLOWBLINK_CONTINUES);
                else printHelp();
            }
            else if(strcmp(argv[2],"service1")==0)
            {
                if(strcmp(argv[3],"on")==0) ATP_BHAL_SetLedState(BHAL_LED_Service1,BHAL_LED_STATE_ON);
                else if(strcmp(argv[3],"off")==0) ATP_BHAL_SetLedState(BHAL_LED_Service1,BHAL_LED_STATE_OFF);
//                else if(strcmp(argv[3],"blink")==0) ATP_BHAL_SetLedState(BHAL_LED_Service1,BHAL_LED_STATE_SLOWBLINK_CONTINUES); //// doesn't work - todo
                else printHelp();
            }
            else if(strcmp(argv[2],"service2")==0)
            {
                if(strcmp(argv[3],"on")==0) ATP_BHAL_SetLedState(BHAL_LED_Service2,BHAL_LED_STATE_ON);
                else if(strcmp(argv[3],"off")==0) ATP_BHAL_SetLedState(BHAL_LED_Service2,BHAL_LED_STATE_OFF);
//                else if(strcmp(argv[3],"blink")==0) ATP_BHAL_SetLedState(BHAL_LED_Service2,BHAL_LED_STATE_SLOWBLINK_CONTINUES); //// doesn't work - todo
                else printHelp();
            }
//////
            else printHelp();
        }
        else printHelp();
    }
    else printHelp();

	return 0;
}
